package com.ekexiu.portal.dao;

import java.sql.Connection;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.util.ArrayList;
import java.util.List;

import org.jfw.apt.annotation.Nullable;
import org.jfw.apt.orm.annotation.dao.DAO;
import org.jfw.apt.orm.annotation.dao.method.From;
import org.jfw.apt.orm.annotation.dao.method.OrderBy;
import org.jfw.apt.orm.annotation.dao.method.operator.DeleteWith;
import org.jfw.apt.orm.annotation.dao.method.operator.Insert;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectList;
import org.jfw.apt.orm.annotation.dao.method.operator.SelectOne;
import org.jfw.apt.orm.annotation.dao.method.operator.Update;
import org.jfw.apt.orm.annotation.dao.method.operator.UpdateWith;
import org.jfw.apt.orm.annotation.dao.param.Set;
import org.jfw.util.PageQueryResult;

import com.ekexiu.portal.po.Professor;
import com.ekexiu.portal.po.Resource;

@DAO
public abstract class ResourceDao {
	@Insert
	public abstract int insert(Connection con,Resource resource) throws SQLException;
	
	@Update
	public abstract int update(Connection con,Resource resource) throws SQLException;
	
	@UpdateWith
	@From(Resource.class)
	public abstract int updateSubject(Connection con, String resourceId, @Set String subject) throws SQLException;

	@UpdateWith
	@From(Resource.class)
	public abstract int updateIndustry(Connection con, String resourceId, @Set String industry) throws SQLException;
	
	@UpdateWith
	@From(Resource.class)
	public abstract int updateDescp(Connection con, String resourceId, @Set String descp) throws SQLException;
	
	@UpdateWith
	@From(Resource.class)
	public abstract int updateHopePayMethod(Connection con, String resourceId, @Set String hopePayMethod) throws SQLException;
	
	@UpdateWith
	@From(Resource.class)
	public abstract int updateCooperationNotes(Connection con,String resourceId,@Set String cooperationNotes) throws SQLException;
	
	@Nullable
	@SelectOne
	public abstract Resource query(Connection con, String resourceId) throws SQLException;
	
	@SelectList
	public abstract List<Resource> query(Connection con) throws SQLException;
	
	@SelectList
	public abstract List<Resource> queryPro(Connection con, String professorId) throws SQLException;
	
	@SelectList
	@OrderBy(" ORDER BY CREATE_TIME ")
	public abstract List<Resource> queryOrder(Connection con, String professorId) throws SQLException;
	
	@DeleteWith
	@From(Resource.class)
	public abstract int delete(Connection con, String resourceId) throws SQLException;
	
	/**
	 * 分页查找所有资源，按资源名称排序
	 * @param key 查询内容（搜索框搜索内容-模糊查询）
	 * @param subject 学术领域
	 * @param industry 研究方向
	 * @param pageSize 
	 * @param pageNo
	 * @return
	 * @throws SQLException
	 */
	public PageQueryResult<Resource> queryPage(Connection con, @Nullable String key, String subject, String industry, int pageSize, int pageNo) throws SQLException {
		PageQueryResult<Resource> _result = new PageQueryResult<Resource>();
		StringBuilder sql = new StringBuilder();
		boolean hasKey = null != key;
		if (hasKey) {
			sql.append(
					" WHERE (( RESOURCE_NAME LIKE ? ) OR ( SUPPORTED_SERVICES LIKE ? ) OR ( DESCP LIKE ? ) OR ( COOPERATION_NOTES LIKE ? ))");
		}
		boolean hasSubject = null != subject;
		boolean hasIndustry = null != industry;
		if (hasSubject) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(SUBJECT LIKE ?)");
		}
		if (hasIndustry) {
			sql.append(sql.length() > 0 ? " AND " : " WHERE ").append("(INDUSTRY LIKE ?)");
		}
		StringBuilder whereSql = sql;
		sql = new StringBuilder();
		sql.append("SELECT COUNT(1) FROM RESOURCE");
		if (whereSql.length() > 0) {
			sql.append(whereSql);
		}
		PreparedStatement ps = con.prepareStatement(sql.toString());
		int paramIndex = 1;
		int totalSize = 0;
		try {
			if (hasKey) {
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
			}
			if (hasSubject) {
				ps.setString(paramIndex++, subject);
			}
			if (hasIndustry) {
				ps.setString(paramIndex++, industry);
			}
			_result.setPageSize(pageSize);
			ResultSet _pageRs = ps.executeQuery();
			try {
				_pageRs.next();
				totalSize = _pageRs.getInt(1);
			} finally {
				try {
					_pageRs.close();
				} catch (Exception _m_6) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception _m_7) {
			}
		}
		_result.setTotal(totalSize);
		if (0 == totalSize) {
			_result.setPageNo(1);
			_result.setData(java.util.Collections.<com.ekexiu.portal.po.Resource> emptyList());
			return _result;
		}
		paramIndex = 1;
		if (1 == pageNo) {
			_result.setPageNo(1);
			sql = new StringBuilder();
			//左连接查询资源表和专家表，方便前台显示专家姓名
			sql.append(" SELECT RESOURCE_ID,RESOURCE_NAME,SUPPORTED_SERVICES,R.DESCP,PROFESSOR_ID,P.NAME,R.CREATE_TIME,R.MODIFY_TIME,HOPE_PAY_METHOD,COOPERATION_NOTES,R.SUBJECT,R.INDUSTRY FROM RESOURCE R LEFT JOIN PROFESSOR P ON PROFESSOR_ID = P.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY RESOURCE_NAME ");
			sql.append(" LIMIT ").append(pageSize);
		} else {
			int _pageSize = totalSize / pageSize;
			if (totalSize % pageSize != 0) {
				++_pageSize;
			}
			if (pageNo > _pageSize) {
				pageNo = _pageSize;
			}
			_result.setPageNo(pageNo);
			--pageNo;
			int _m_10 = (pageNo * pageSize);
			sql = new StringBuilder();
			sql.append(" SELECT RESOURCE_ID,RESOURCE_NAME,SUPPORTED_SERVICES,R.DESCP,PROFESSOR_ID,P.NAME,R.CREATE_TIME,R.MODIFY_TIME,HOPE_PAY_METHOD,COOPERATION_NOTES,R.SUBJECT,R.INDUSTRY FROM RESOURCE R LEFT JOIN PROFESSOR P ON PROFESSOR_ID = P.ID ");
			if (whereSql.length() > 0) {
				sql.append(whereSql);
			}
			sql.append(" ORDER BY RESOURCE ");
			sql.append(" LIMIT ").append(pageSize).append(" OFFSET ").append(_m_10);
		}
		ps = con.prepareStatement(sql.toString());
		try {
			if (hasKey) {
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
				ps.setString(paramIndex++, key);
			}
			if (hasSubject) {
				ps.setString(paramIndex++, subject);
			}
			if (hasIndustry) {
				ps.setString(paramIndex++, industry);
			}
			ResultSet rs = ps.executeQuery();
			try {
				List<Resource> _m_11 = new ArrayList<Resource>();
				_result.setData(_m_11);
				while (rs.next()) {
					Resource obj = new Resource();
					obj.setResourceId(rs.getString(1));
					String _m_13 = rs.getString(2);
					if (rs.wasNull()) {
						_m_13 = null;
					}
					obj.setResourceName(_m_13);
					String _m_14 = rs.getString(3);
					if (rs.wasNull()) {
						_m_14 = null;
					}
					obj.setSupportedServices(_m_14);
					String _m_15 = rs.getString(4);
					if (rs.wasNull()) {
						_m_15 = null;
					}
					obj.setDescp(_m_15);
					Professor professor = new Professor();
					professor.setId(rs.getString(5));
					professor.setName(rs.getString(6));
					obj.setProfessor(professor);
					obj.setCreateTime(rs.getString(7));
					obj.setModifyTime(rs.getString(8));
					obj.setHopePayMethod(rs.getString(9));
					String _m_16 = rs.getString(10);
					if (rs.wasNull()) {
						_m_16 = null;
					}
					obj.setCooperationNotes(_m_16);
					String _m_17 = rs.getString(11);
					if (rs.wasNull()) {
						_m_17 = null;
					}
					obj.setSubject(_m_17);
					String _m_18 = rs.getString(12);
					if (rs.wasNull()) {
						_m_18 = null;
					}
					obj.setIndustry(_m_18);
					_m_11.add(obj);
				}
				return _result;
			} finally {
				try {
					rs.close();
				} catch (Exception _m_19) {
				}
			}
		} finally {
			try {
				ps.close();
			} catch (Exception _m_20) {
			}
		}
	}
	
}
